//--------------------------------------------------------------------------//
// iq . 2007 . kindernoiser 4k intro demo by RGBA                           //
//--------------------------------------------------------------------------//

#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#include <windows.h>
#include <GL/gl.h>
#include <math.h>
#include "main.h"
#include "sys/libc.h"
#include "ext.h"
#include "fsh_julia3d.inl"
#include "fsh_ssao.inl"
#include "fsh_final.inl"
#include "fsh_glow.inl"
#include "vsh_2d.inl"
#include "fp.h"
//=================================================================================================================
//=================================================================================================================
#define NUMPASSES  5
#define NUMSHADERS 5

#define TX0ID  2
#define TX1ID  3
#define TX2ID  4
#define ZBUID  5
#define FBOID  6

// 0.pos { x,y,z,ite }
// 1.tar { x,y,z,fade }
// 2.jul { x,y,z,w   }
static float mo[12] = { p0d00, p0d00, p0d00, 11.0f, 
                        p0d00, p0d00, p0d00, p0d00, 
                       -p0d30, p0d00, p0d00, p0d00 };
static float ma[12] = { p1d10, p1d00, p1d10, p0d00, 
                        p0d10, p0d10, p0d10, p0d00, 
                        p0d60, p0d55, p0d55, p0d55 };
static float mf[12] = { p0d10, p0d10, p0d10, p0d00, 
                        p0d30, p0d30, p0d30, p0d00, 
                        p0d31, p0d29, p0d33, p0d27 };
static float mp[12] = { p1d60, p0d30, p0d00, p0d00, 
                        p0d10, p0d40, p0d70, p0d00, 
                        p2d10, p1d00, p2d00, 5.0f };

static const float tof[2] = { 603.0f, 944.0f };
static const char *fshaders[NUMSHADERS] = { fsh_julia3D, fsh_ssao, fsh_final, fsh_hblur, fsh_vblur };

//=================================================================================================================
//=================================================================================================================

static int pid[NUMPASSES];
static float fparams[ 4*4];
static float fkernef[32*4];
#ifdef CONFIGDIALOG
static unsigned int XRES;
static unsigned int YRES;
#endif

//=================================================================================================================
//=================================================================================================================

static void initShader( int *pid, const char *vs, const char *fs )
{
    pid[0] = oglCreateProgram();                                // 0x8000001+ 
    const int vsId = oglCreateShader( GL_VERTEX_SHADER );       // 0x4000001+
    const int fsId = oglCreateShader( GL_FRAGMENT_SHADER );     // 0x2000001+
    oglShaderSource( vsId, 1, &vs, 0 );
    oglShaderSource( fsId, 1, &fs, 0 );
    oglCompileShader( vsId );
    oglCompileShader( fsId );
    oglAttachShader( pid[0], fsId );
    oglAttachShader( pid[0], vsId );
    oglLinkProgram( pid[0] );
            #ifdef DEBUG
            int     result;
            char    info[2048];
            oglGetObjectParameteriv( vsId, GL_OBJECT_COMPILE_STATUS_ARB, &result );
            oglGetInfoLog( vsId, 1024, NULL, (char *)info );
            if( !result )
                 DebugBreak();
            oglGetObjectParameteriv( fsId, GL_OBJECT_COMPILE_STATUS_ARB, &result );
            oglGetInfoLog( fsId, 1024, NULL, (char *)info );
            if( !result ) 
                 DebugBreak();
            oglGetObjectParameteriv( pid[0], GL_OBJECT_LINK_STATUS_ARB, &result );
            oglGetInfoLog( pid[0], 1024, NULL, (char*)info );
            if( !result ) 
                 DebugBreak();
            #endif
}

#ifdef CONFIGDIALOG
int intro_init( const unsigned int *res )
{
    XRES = res[0];
    YRES = res[1];
#else
int intro_init( void )
{
#endif
    int i;
    // extensions
    if( !EXT_Init() )
        return( 0 );
    // kernel
    int mirand = 0;
    for( i=0; i<(32*4); i++ ) 
    {
        mirand = mirand*0x343fd+0x269ec3; int a = (mirand>>16)&32767;
        fkernef[i] = -1.0f + (2.0f/32768.0f)*(float)a; 
    }
    // textures
    for( i=0; i<3; i++ )
    {
        glBindTexture( GL_TEXTURE_2D, TX0ID+i );
        glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA16F_ARB, XRES, YRES, 0, GL_RGBA, GL_FLOAT, 0 );
        glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
        glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
        glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     GL_CLAMP_TO_EDGE );
        glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     GL_CLAMP_TO_EDGE );
        glBindTexture( GL_TEXTURE_2D, 0 );
    }
    // depth renderbuffer
    oglBindRenderbufferEXT(GL_RENDERBUFFER_EXT, ZBUID );
    oglRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24, XRES, YRES );
    oglBindRenderbufferEXT(GL_RENDERBUFFER_EXT, 0 );
    // shaders
    for( i=0; i<NUMSHADERS; i++ )
        initShader( pid+i, vsh_2d, fshaders[i] );
    return 1;
}

//=================================================================================================================
//=================================================================================================================
//
// 0: x,x -> 0(tno)
// 1: 0,x -> 1(ao)
// 2: 0(tno),1(ao) -> 2 (fin)
// 3: 2(fin),x -> 1(hbl)
// 4: 2(hbl),1(fin) -> x (vbl)
//                                         julia  ao     final  hblur  vblur
static const unsigned char pass_t0id[] = { 0,     TX0ID, TX0ID, TX2ID, TX1ID };
static const unsigned char pass_t1id[] = { 0,     0,     TX1ID, 0,     TX2ID };
static const unsigned char pass_targ[] = { TX0ID, TX1ID, TX2ID, TX1ID, 0     };

static const unsigned char kItes[] = { 
      3, 3, 3, 3, 3, 3, 3, 3,
      2,11, 3,12,12,12,12,12,
     12,12,12,12,12,12,12,12,
     12,12,12 };

#define ZE 0
#define ON 1
#define UN 1
static const unsigned char kSeq[] = {
    ZE,ZE,ZE,ZE,ZE,ZE,ZE,ZE,
    ZE,ON,ZE,UN,ZE,ZE,ON,ZE,
    ZE,ZE,ZE,ON,ZE,ZE,ZE,ZE,
    ZE,ZE,ON,ON
};

static const unsigned short tKeys[] = {
     400, 865, 873, 889, 912, 955,1030,1240,
    1310,1810,2050,2550,3085,3600,4100,4860,
    5100,5630,6550,6650,7070,8065,8245,8450,
    8755,9600,10400
};


static const signed char fadeS[] = { 
    2,0,0,0,0,0,0,-2,
    0,-5,0,1,0,0,-4,0,
    0,0,0,0,0,0,0,0,
    0,0,-7,0 };

static const signed char fadeO[] = { 
    0,1,1,1,1,1,1,1,
    0,1,1,1,1,1,1,1,
    1,1,1,1,1,1,1,1,
    1,1,1,0 };

void intro_do( long time )
{
    int i, s;
    //--- update parameters -----------------------------------------
    for( s=0; time>(tKeys[s]*10); s++ ) {}

    float t  = 0.001f*(float)time;
    float rt = 0.001f*(float)(time-tKeys[s-1]*10);
    
    t = t + tof[kSeq[s]] * (float)s;

    mo[3] = (float)kItes[s];

    for( i=0; i<12; i++ ) fparams[i] = mo[i] + ma[i]*cosf( mf[i]*t + mp[i] );

    float fade = (float)fadeO[s]; if( fadeS[s] ) fade+=  rt/(float)fadeS[s];
    if( fade>1.0f ) fade = 1.0f;

    float ba = p0d10;
    if( s==1 ) ba= 1.0f;
    if( s==4 ) ba= 1.0f;
    if( s==9 ) ba=-1.0f;
    fparams[12] = p2d00-p0d90*ba;
    fparams[13] = p0d80-p0d30*ba;
    fparams[14] = p0d90-p0d70*ba;
    fparams[15] = p0d90+p0d50*ba;
    if( s==9 ) fparams[15] = 4.0f;
    if( s==20 || s==22 ) fparams[15] = 1.5f;

    fparams[14] *= fade;
    fparams[15] *= fade;

    fparams[12] /= (float)XRES; // blur kernel width...
    //--- render -----------------------------------------

    oglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, FBOID );

    for( i=0; i<NUMPASSES; i++ )
    {
        oglUseProgram( pid[i] );
        oglUniform4fv( oglGetUniformLocation( pid[i], "fpar" ),  4, fparams );
        oglUniform4fv( oglGetUniformLocation( pid[i], "fk3f" ), 32, fkernef );
        oglUniform1i(  oglGetUniformLocation( pid[i], "tex0" ), 0);
        oglUniform1i(  oglGetUniformLocation( pid[i], "tex1" ), 1);
        oglActiveTextureARB( GL_TEXTURE0_ARB ); glBindTexture( GL_TEXTURE_2D, pass_t0id[i] );  // zbufer+normal
        oglActiveTextureARB( GL_TEXTURE1_ARB ); glBindTexture( GL_TEXTURE_2D, pass_t1id[i] );  // noise
        if( i==(NUMPASSES-1) )  // last frame, render to color buffer
            oglBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0 );
        else
        {
            oglFramebufferTexture2DEXT(    GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D,       pass_targ[i], 0);
            oglFramebufferRenderbufferEXT( GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT,  GL_RENDERBUFFER_EXT, ZBUID );
        }
        glRects( -1, -1, 1, 1 );
    }
}