#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#include <windows.h>
#include "mzk.h"
#include "song.h"

#define DOECHO

//=================================================================================================================
//=================================================================================================================

static char *paths[2] = { "drivers/gm.dls", "drivers/etc/gm.dls" };

static unsigned char volumenes[NUMCHANNELS] = { 6, 6, 6, 6, 6, 9, 9, 9, 6 };
#ifdef DOECHO
static unsigned char echosv[NUMCHANNELS] = { 4, 4, 4, 4, 4, 4, 4, 1, 4 };
#endif

//=================================================================================================================
//=================================================================================================================

static short        sampleBuffer[44100*3];
static int          ibuffer[MZK_NUMSAMPLESC];

//=================================================================================================================
//=================================================================================================================

// y = 2^x
static __forceinline float m2powf( float f )
{
  __asm
  {
    fld   dword ptr [f]
    fld1
    fld   st(1)
    fprem
    f2xm1
    faddp st(1), st
    fscale
    fstp  st(1)
    fstp  dword ptr [f]
  }

  return f;
}


void mzk_init( short *buffer )
{
    int             m, p;
    int             sampleLoops; 
    int             lastmuestra;
    HANDLE          h;
    unsigned long   wr;

    // open sample file
    for( h=INVALID_HANDLE_VALUE, m=0; h==INVALID_HANDLE_VALUE; m++ )
        h = (HANDLE)OpenFile( paths[m], (OFSTRUCT*)sampleBuffer, OF_READ );

    // render "song"
    //mzeroset( ibuffer, MZK_NUMSAMPLESC*sizeof(int) );
    const unsigned char *deltas_ptr = mzk_deltas_b0;
    const unsigned char *notas_ptr  = mzk_notas;

    for( int c=0; c<NUMCHANNELS; c++ )
    {
        // read sample
        SetFilePointer( h, mzk_sampleOffsets[c], 0, FILE_BEGIN );
        ReadFile( h, &sampleLoops, 4, &wr, 0 );
        ReadFile( h, sampleBuffer, sampleLoops, &wr, 0 );
        sampleLoops = sampleLoops/2;

        // mirror, and envelope for sample 1
        if( c==1 )
        {
            for( m=0; m<sampleLoops; m++ ) 
            {
                int a = 4096; if( m>5000 ) a = 4096 - 4096*(m-5000)/(sampleLoops-5000);
                sampleBuffer[sampleLoops+m] = (sampleBuffer[sampleLoops-1-m]*a)>>12;
            }
            sampleLoops*=2;
        }

        // render notes
        lastmuestra = 0;
        const unsigned int nd = (unsigned int)mzk_numdeltas[c];
        for( unsigned int d=0; d<nd; d++ )
        {   
            const int delta = ((deltas_ptr[NUMNOTAS]<<8)+deltas_ptr[0]) * SPT; deltas_ptr++;
            const int nota  = *notas_ptr++;
            const int freq  = (int)(30.02f*m2powf( (float)nota/12.0f ) );

            lastmuestra += delta;

            p = 0;
            m = lastmuestra;
            do
            {
                // linear interpolation
                const int ipos = p>>10;
                const int fpos = p&1023;
                const int a = sampleBuffer[ipos+0];
                const int b = sampleBuffer[ipos+1];
                const int f = a + ((fpos*(b-a))>>10);

                // add to buffer
                #ifdef DEBUG
                    int tm = m - MZK_START*MZK_RATE;
                    if( tm>=0 ) 
                    {
                    ibuffer[tm] += f * volumenes[c];
                    #ifdef DOECHO
                    ibuffer[tm+8192] += (f*volumenes[c])>>echosv[c];
                    #endif
                    }
                #else
                    ibuffer[m] += f * volumenes[c];
                    #ifdef DOECHO
                    ibuffer[m+8192] += (f*volumenes[c])>>echosv[c];
                    #endif
                #endif

                m++;
                p += freq;
            }while( (p>>10)<sampleLoops && m<(MZK_RATE*MZK_DURATION) );
        }

    }



    //----------------------------

#if 0
    for( m=0; m<MZK_NUMSAMPLES; m++ ) 
    {
        int ix = ibuffer[m] / 16;
        if( ix>32767 ) ix=32767; else if( ix<-32767 ) ix=-32767;
        buffer[m] = ix;
    }
#else

    _asm
    {
        lea esi, [ibuffer]
        mov edi, buffer
        mov ecx, MZK_NUMSAMPLES
        mov ebx, 32767
        myloop:
            lodsd
            sar     eax, 4
            cmp     eax, ebx
            cmovg   eax, ebx
            neg     eax
            cmp     eax, ebx
            cmovg   eax, ebx
            neg     eax
            stosw
        dec     ecx
        jnz     myloop
    }

#endif

//    CloseHandle( h );
}
