//--------------------------------------------------------------------------//
// iq . 2007 . kindernoiser 4k intro demo by RGBA                           //
//--------------------------------------------------------------------------//

#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#include <windows.h>
#include <mmsystem.h>
#include "../../intro.h"
#include "../../mzk.h"
#include "../../main.h"

//----------------------------------------------------------------------------

static HDC     hDC;
static HGLRC   hRC;
static HWND    hWnd;
static short   myMuzik[MZK_NUMSAMPLES+22];
#ifdef CONFIGDIALOG
static int     dlgResoID;
static int     dlgFulls;
#endif

//----------------------------------------------------------------------------
#ifdef __cplusplus
extern "C" 
{
#endif
int  _fltused = 0;
#ifdef __cplusplus
}
#endif

static PIXELFORMATDESCRIPTOR pfd = {
    sizeof(PIXELFORMATDESCRIPTOR),
    1,
    PFD_DRAW_TO_WINDOW|PFD_SUPPORT_OPENGL|PFD_DOUBLEBUFFER,
    PFD_TYPE_RGBA,
    32, 0, 0, 0, 0, 0, 0, 
    0,  0, 0, 0, 0, 0, 0,
    32, 
    0, 
    0,
    PFD_MAIN_PLANE,
    0, 0, 0, 0 };

static int wavHeader[11] = {
    0x46464952, 
    MZK_NUMSAMPLES*2+36, 
    0x45564157, 
    0x20746D66, 
    16, 
    WAVE_FORMAT_PCM|(MZK_NUMCHANNELS<<16), 
    MZK_RATE, 
    MZK_RATE*MZK_NUMCHANNELS*sizeof(short), 
    (MZK_NUMCHANNELS*sizeof(short))|((8*sizeof(short))<<16),
    0x61746164, 
    MZK_NUMSAMPLES*sizeof(short) };

//----------------------------------------------------------------------------
#ifdef CONFIGDIALOG
__forceinline int window_init( const unsigned int *res )
#else
__forceinline int window_init( void )
#endif
{
    DEVMODE	dmScreenSettings;
    #ifdef CONFIGDIALOG
    if( dlgFulls )
    #endif
    {
        dmScreenSettings.dmSize       = sizeof(DEVMODE);
        dmScreenSettings.dmFields     = DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;
        dmScreenSettings.dmBitsPerPel = 32;
        #ifdef CONFIGDIALOG
        dmScreenSettings.dmPelsWidth  = res[0];
        dmScreenSettings.dmPelsHeight = res[1];
        #else
        dmScreenSettings.dmPelsWidth  = XRES;
        dmScreenSettings.dmPelsHeight = YRES;
        #endif
        if( ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
            return( 0 );
        ShowCursor(0);
    }

    #ifdef CONFIGDIALOG

    #ifdef USELOADING
    const int style = SS_CENTER|SS_CENTERIMAGE|WS_POPUP|WS_VISIBLE| ((!dlgFulls)*(WS_EX_WINDOWEDGE|WS_CAPTION|WS_SYSMENU|WS_CLIPCHILDREN));
    #else
    const int style = WS_POPUP|WS_VISIBLE| ((!dlgFulls)*(WS_EX_WINDOWEDGE|WS_CAPTION|WS_SYSMENU|WS_CLIPCHILDREN));
    #endif
    RECT rec = { 0, 0, res[0], res[1] };
    AdjustWindowRect( &rec, style, 0 );
    HWND hWnd = CreateWindow( "static","rgba :: kindernoiser",style,0,0,rec.right-rec.left,rec.bottom-rec.top,0,0,0,0);
    #else
    #ifdef USELOADING
    HWND hWnd = CreateWindow( "static","rgba :: kindernoiser",WS_POPUP|WS_VISIBLE|SS_CENTER|SS_CENTERIMAGE,0,0,XRES,YRES,0,0,0,0);
    #else
    HWND hWnd = CreateWindow( "static",0,WS_POPUP|WS_VISIBLE,0,0,XRES,YRES,0,0,0,0);
    #endif
    #endif
    if( !hWnd ) return( 0 );

    #ifdef USELOADING
    SendMessage( hWnd, WM_SETFONT, (WPARAM)CreateFont(44,0,0,0,0,0,0,0,0,0,0,ANTIALIASED_QUALITY,0,"tahoma"), TRUE );
    #endif


    hDC = GetDC(hWnd);
    //if( !hDC ) return( 0 );

    unsigned int PixelFormat = ChoosePixelFormat(hDC,&pfd);
    if( !PixelFormat ) return( 0 );

    if( !SetPixelFormat(hDC,PixelFormat,&pfd) ) return( 0 );

    hRC = wglCreateContext(hDC);
    if( !hRC ) return( 0 );

    //if( !wglMakeCurrent(hDC,hRC) ) return( 0 );
    wglMakeCurrent(hDC,hRC);
    
    return( 1 );
}

__forceinline void window_end( void )
{
    #ifdef CLEANDESTROY
    if( hRC )
    {
        wglMakeCurrent( 0, 0 );
        wglDeleteContext( hRC );
    }
    if( hDC  ) ReleaseDC( hWnd, hDC );
    if( hWnd ) DestroyWindow( hWnd );
    #endif

    ChangeDisplaySettings( 0, 0 );
    ShowCursor(1);
}


//===============================================
#ifdef CONFIGDIALOG

static const char resoss[] =  
  " 320 x  240\x0"
  " 640 x  480\x0"
  " 800 x  600\x0"
  "1024 x  768\x0"
  "1280 x  960\x0"
  "1280 x 1024\x0"
  "1600 x 1200";

static const unsigned int resos[] = {
   320,  240,
   640,  480,
   800,  600,
  1024,  768,
  1280,  960,
  1280, 1024,
  1600, 1200 };

#define NUMRESOS (sizeof(resos)/8)

static unsigned char dlgData[] = {
    0xc0, 0x08, 0xc0, 0x90, 0x00, 0x00, 0x00, 0x00, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x73, 0x00, 
    0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x6b, 0x00, 0x69, 0x00, 0x6e, 0x00, 0x64, 0x00, 0x65, 0x00, 
    0x72, 0x00, 0x6e, 0x00, 0x6f, 0x00, 0x69, 0x00, 0x73, 0x00, 0x65, 0x00, 0x72, 0x00, 0x20, 0x00, 
    0x5b, 0x00, 0x72, 0x00, 0x67, 0x00, 0x62, 0x00, 0x61, 0x00, 0x20, 0x00, 0x32, 0x00, 0x30, 0x00, 
    0x30, 0x00, 0x37, 0x00, 0x5d, 0x00, 0x00, 0x00, 0x08, 0x00, 0x4d, 0x00, 0x53, 0x00, 0x20, 0x00, 
    0x53, 0x00, 0x61, 0x00, 0x6e, 0x00, 0x73, 0x00, 0x20, 0x00, 0x53, 0x00, 0x65, 0x00, 0x72, 0x00, 
    0x69, 0x00, 0x66, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x50, 0x00, 0x00, 0x00, 0x00, 
    0x05, 0x00, 0x0a, 0x00, 0x3c, 0x00, 0x20, 0x00, 0xff, 0xff, 0xff, 0xff, 0x82, 0x00, 0x72, 0x00, 
    0x65, 0x00, 0x73, 0x00, 0x6f, 0x00, 0x6c, 0x00, 0x75, 0x00, 0x74, 0x00, 0x69, 0x00, 0x6f, 0x00, 
    0x6e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x21, 0x50, 0x00, 0x00, 0x00, 0x00, 
    0x05, 0x00, 0x14, 0x00, 0x3c, 0x00, 0x40, 0x00, 0x6f, 0x00, 0xff, 0xff, 0x85, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x01, 0x50, 0x00, 0x00, 0x00, 0x00, 0x05, 0x00, 0x28, 0x00, 
    0x3c, 0x00, 0x0a, 0x00, 0x70, 0x00, 0xff, 0xff, 0x80, 0x00, 0x77, 0x00, 0x69, 0x00, 0x6e, 0x00, 
    0x64, 0x00, 0x6f, 0x00, 0x77, 0x00, 0x65, 0x00, 0x64, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
    0x01, 0x00, 0x01, 0x50, 0x00, 0x00, 0x00, 0x00, 0x05, 0x00, 0x3c, 0x00, 0x32, 0x00, 0x0d, 0x00, 
    0x01, 0x00, 0xff, 0xff, 0x80, 0x00, 0x26, 0x00, 0x67, 0x00, 0x6f, 0x00, 0x21, 0x00, 0x00, 0x00, 
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x50, 0x00, 0x00, 0x00, 0x00, 0x3c, 0x00, 0x3c, 0x00, 
    0x32, 0x00, 0x0d, 0x00, 0x02, 0x00, 0xff, 0xff, 0x80, 0x00, 0x65, 0x00, 0x26, 0x00, 0x78, 0x00, 
    0x69, 0x00, 0x74, 0x00, 0x00, 0x00, 0x00, 0x00 };

#define IDC_DIALOG          110
#define IDC_CHECK1          111
#define IDC_CHECK2          112

static BOOL CALLBACK myDlgProc( HWND dlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
    HWND hcombo = GetDlgItem( dlg, IDC_CHECK1 );

    if( msg==WM_INITDIALOG )
    {
        //SendMessage(hcombo, CB_RESETCONTENT, 0, 0);
        for( int i=0; i<NUMRESOS; i++ ) 
            SendMessage(hcombo, CB_INSERTSTRING, -1, (LPARAM)(resoss+12*i));
        SendMessage(hcombo, CB_SETCURSEL, 2, 0);
    }
    else if( msg==WM_COMMAND )
    {
        if( wParam == IDOK )
        {
            dlgResoID = SendMessage(hcombo, CB_GETCURSEL, 0, 0);
            dlgFulls = !IsDlgButtonChecked( dlg, IDC_CHECK2 );
            EndDialog( dlg, 1 );
        }
        else if( wParam == IDCANCEL )
        {
            EndDialog( dlg, 0 );
        }
    }

    return( 0 );
}
#endif

void entrypoint( void )
{
    long        to=0;

    #ifdef CONFIGDIALOG
    //HINSTANCE h = GetModuleHandle(0);
    //if( !DialogBoxIndirect( h ,(LPDLGTEMPLATE)dlgData,0,myDlgProc) ) return;
    if( !DialogBoxIndirect( 0, (LPDLGTEMPLATE)dlgData, 0, myDlgProc) ) return;
    if( !window_init(resos+2*dlgResoID) ) goto endend;
    if( !intro_init( resos+2*dlgResoID) ) goto endend;
    #else
    // create window
    if( !window_init() ) goto endend;
    if( !intro_init()  ) goto endend;
    #endif

    intro_do( 0 );

    // mzk
    mzk_init( myMuzik+22 );
    memcpy( myMuzik, wavHeader, 44 );

    #ifdef USELOADING
    Sleep( 3000 );
    #endif

    sndPlaySound( (const char*)&myMuzik, SND_ASYNC|SND_MEMORY );


    for(;;)
    {
        MSG msg;
        PeekMessage(&msg,0,0,0,PM_REMOVE);
        if( msg.message==WM_KEYDOWN    && msg.wParam==VK_ESCAPE ) break;   // ESC
      //if( msg.message==WM_SYSCOMMAND && msg.wParam==SC_CLOSE  ) done = 1;   // ALT F4
        long t = timeGetTime(); if( !to ) to=t; t=t-to;
        intro_do( t );
        if( t>(MZK_DURATION*1000) ) break;
        //SwapBuffers( hDC );
        wglSwapLayerBuffers( hDC, WGL_SWAP_MAIN_PLANE );
    }

    sndPlaySound(0,0);

endend:
    window_end();

    ExitProcess(0);
}
